<?php

/******************************************************************************
import-haloscan.php
Copyright (C) 2005 Justin Watt

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


This code takes the import-blogger.php script from Wordpress v1.5.1.2
and merges it with some code originally written by Phil Ringnalda to 
parse CAIF comments, which he describes here:
http://philringnalda.com/blog/2002/05/importing_yaccs_comments_in_movable_type.php

The code assumes that this file will live in the wordpress/wp-admin directory
and that the CAIF files to be imported with live in the wordpress directory.

Haloscan allows exporting of CAIF (Comment Archive and Interchange Format)
files in 1000 comment blocks, so if you have more than 1000 comments, save
the CAIF files as export1.xml, export2.xml, and so on. This script will read
all files in the wordpress directory that begin with the "export" and attempt
to parse them according to the CAIF xml format.

This script also assumes that the thread_id for each group of comments related
to a post is located inside an html comment <!-- ---> as the beginning of the 
body of each wordpress post. If the script does not find a thread id, it will 
skip that post and move on to the next.

More information on using this code can be found at the following blog post:
http://justinsomnia.org/2005/06/importing-haloscan-comments-into-wordpress/

Enjoy.
Justin Watt
justincwatt@gmail.com
http://justinsomnia.org/
******************************************************************************/

require_once('../wp-config.php');

header( 'Content-Type: text/html; charset=utf-8' );

$action = $_REQUEST['action'];

switch ($action) {

case "step1":
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
	<title>Haloscan to WordPress - Converting...</title>
	<link rel="stylesheet" href="wp-admin.css" type="text/css">
</head>
<body>
<div class="wrap">
<h1>Haloscan to WordPress</h1>
<p>The importer is running...</p>

<?php


// initialize some variables
$blogger_to_wordpress = array();
$comment_import_count = 0;

// get wordpress id and post content from every post
$posts = $wpdb->get_results("SELECT ID, post_content FROM $wpdb->posts");

// loop through each wordpress post
foreach ($posts as $post) 
{
    // get the post id
    $wordpress_post_id = $post->ID;

    // trim the content of spaces
    $post_content = trim($post->post_content);
    
    // get the blogger post id embedded within an html comment,
    // add to blogger_to_wordpress array relating blogger_post_id to wordpress_post_id
    if (substr($post_content, 0, 4) == "<!--")
    {
        $end_comment_position = strpos($post_content, "-->");
        $blogger_post_id = substr($post_content, 4, $end_comment_position-4);
        $blogger_to_wordpress[$blogger_post_id] = $wordpress_post_id;
    }
}

// loop through each file beginning with export in the parent dir
// and call the parse_caif function
if ($handle = opendir('..')) 
{
   while (false !== ($filename = readdir($handle))) 
   {
       if (substr($filename,0,6) == "export")
       {
           // call parse_caif function below
           parse_caif("../" . $filename);
       }
   }
   closedir($handle);
}
?>

<strong>Done. <?php echo $comment_import_count ?> comments imported.</strong>

<p>&nbsp;</p>
<p>Completed Haloscan to WordPress import!</p>
<p>Now you can go and <a href="../wp-login.php">log in</a>, have fun!</p>
</div>
</body>
</html>
	<?php
	break;

default:

	?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
	<title>Haloscan to WordPress Import Utility</title>
	<link rel="stylesheet" href="wp-admin.css" type="text/css">
</head>

<body>
<div class="wrap">
<h1>Haloscan to WordPress</h1>
<p>This is a basic Haloscan to WordPress import script. It has been tested to work with a Blogger blog that was imported to Wordpress.</p>
<p>What it does:</p>
<ul>
	<li>Parses your Haloscan <acronym title="Comment Archive and Interchange Format">CAIF</acronym> export files to retrieve your comments.</li>
</ul>

<h2>First step: Import Blogger posts</h2>

<p>Import Bloggers posts into WordPress so that the old blogger post id is stored as an HTML comment at the beginning of the post content. See <a href="http://justinsomnia.org/2005/06/importing-haloscan-comments-into-wordpress/">Importing Haloscan comments into WordPress</a> for more details.</p>

<h2>Second step: export Haloscan comments</h2>

Goto <a href="http://haloscan.com/">haloscan.com</a> and login. Goto Manage Comments &gt; Export. There will be an export link for every thousand comments you have. Clicking one of the export links will prompt you to save the file as "export.xml". If there is more than one link (i.e. if you have more than 1000 comments), rename the first file to export1.xml, the second file to export2.xml, and so on. Save the files to your desktop and upload them to your Wordpress directory, e.g. <code>/<em>path/to/wordpress</em>/</code></p>

<form name="stepOne" method="get">
<input type="hidden" name="action" value="step1" />

<h2>Third step: let's click OK:</h2>

<p>When you're ready, click OK to start importing: <input type="submit" name="submit" value="OK" /><br /><br />

<i>Note: the script might take some time. DO NOT STOP IT or else you won't have a complete import.</i></p>
</form>
</div>
</body>
</html>

<?php
	break;
}




// functions for caif parsing
// thanks again to Phil Ringnalda
// who's code these functions were adapated from

function start_element_handler($parser, $name, $attrs='')
{
    global $current_thread, 
           $current_tag, 
           $comment_number, 
           $thread_id;
    
    // keep track of the current tag
    // so character_data_handler knows what to do
    $current_tag = $name;

    switch($name)
    {
        case 'THREAD':
        $thread_id = $attrs['ID'];  // get the thread_id (blogger_post_id) which is an attribute on the thread tag
        $comment_number = -1;       // reset the comment counter
        $current_thread = "";       // reset the thread array container
        break;
        
        case 'COMMENT':
        $comment_number++;          // increment the comment counter
        break;
        
        case 'DATETIME':
        case 'NAME':
        case 'EMAIL':
        case 'URI':
        case 'IP':
        case 'TEXT':

        // initialize cdata value for current tag
        $current_thread[$comment_number][$current_tag] = '';
        default:
        break;
    }
}

function end_element_handler($parser, $name, $attrs='')
{
    global $current_thread, 
           $current_tag;

    switch($name)
    {
        case 'DATETIME':
        case 'NAME':
        case 'EMAIL':
        case 'URI':
        case 'IP':
        case 'TEXT':
            $current_tag = ''; // reset the current tag
        break;
        
        case 'THREAD':
            commit_thread();      // commit comments in the thread to database
            $current_tag = '';    // reset the current tag
            $current_thread = ''; // reset the current thread array
        break;
        
        default:
        break;
    }
}

function character_data_handler($parser, $data)
{
    global $current_tag, 
           $current_thread, 
           $comment_number;
    
    // operate on comment fields
    switch($current_tag)
    {
        case 'DATETIME':
            // format dates for mysql appropriately
            $datestamp = str_replace("T", " ", $data);
            $datestamp = str_replace("-05:00", "", $datestamp);
            $current_thread[$comment_number]['DATETIME'] = $datestamp;
            break;
        
        case 'NAME':
        case 'EMAIL':
        case 'URI':
        case 'IP':
            $current_thread[$comment_number][$current_tag] .= $data;
            break;
        
        case 'TEXT':
            // replace any number of spaces, followed by a newline with a <br /> tag
            $current_thread[$comment_number]['TEXT'] .= preg_replace("/\s*\n/", "<br />", $data);
            break;
        
        default:
            break;
    }
}

		
function commit_thread()
{
    global $current_thread, 
           $thread_id,
           $blogger_to_wordpress,
           $wpdb,
           $comment_import_count;
    
    // loop through each comment in thread
    for ($i = 0; $i < sizeof($current_thread); $i++)
    {
        // if the current thread exists in wordpress
        // import the thread
        if (isset($blogger_to_wordpress[$thread_id]))
        {   
            $comment_post_ID      = $blogger_to_wordpress[$thread_id];
            $comment_author       = addslashes(strip_tags($current_thread[$i]['NAME']));
            $comment_author_email = $current_thread[$i]['EMAIL'];
            $comment_author_url   = addslashes($current_thread[$i]['URI']);
            $comment_author_IP    = $current_thread[$i]['IP'];
            $comment_date         = $current_thread[$i]['DATETIME'];
            $comment_content      = addslashes(str_replace(array('<br>','<BR>','<br/>','<BR/>','<br />','<BR />'), "\n", $current_thread[$i]['TEXT']));
            $comment_approved     = 1;

            $comment = compact('comment_post_ID', 'comment_author', 'comment_author_email', 'comment_author_url', 'comment_author_IP', 'comment_date', 'comment_content', 'comment_approved');
            wp_filter_comment($comment);
            wp_insert_comment($comment);

            $comment_import_count++;
        }
        else
        {
            continue;
        }
    }
}

function parse_caif($filename)
{
    // set up parser for caif
    $type = 'UTF-8';
    $xml_parser = xml_parser_create($type);

    // set basic parser options
    xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, true);
    xml_parser_set_option($xml_parser, XML_OPTION_TARGET_ENCODING, 'UTF-8');

    // set start element, end element, and character data handlers
    xml_set_element_handler($xml_parser, 'start_element_handler','end_element_handler');
    xml_set_character_data_handler($xml_parser, 'character_data_handler');
    
    // get caif file
    if ($fp = @fopen($filename, 'r')) {
        while ($data = fread($fp, 4096)) 
        {
            if (!($data = utf8_encode($data))) 
            {
                echo 'ERROR'."\n";
            }
            
            // parse caif file (die on error)
            if (!xml_parse($xml_parser, $data, feof($fp))) 
            {
                die(sprintf( "XML error: %s at line %d\n\n",
                    xml_error_string(xml_get_error_code($xml_parser)),
                    xml_get_current_line_number($xml_parser)));
            }
        }
    }
    else 
    {
        echo "Couldn't find caif input file";
    }

    xml_parser_free($xml_parser);
}

?>